import Component from '@ember/component'
import { htmlSafe } from '@ember/string'
import { set } from '@ember/object'

import layout from '../../templates/components/en-dropdown/content'
import DocumentEvent from 'en-select/mixins/document-event'

export default Component.extend(DocumentEvent, {
  layout,
  classNames: ['dropdown-content'],
  classNameBindings: [
    'isOpen:is-open',
    'isDisabled:is-disabled',
    'positionFromTop:from-top',
    'positionFromRight:from-right',
  ],
  attributeBindings: ['style'],
  documentEvents: ['scroll', 'resize'],

  style: htmlSafe(''),

  isOpen: false,
  isDisabled: false,

  isFixed: false,
  positionFromTop: false,
  positionFromRight: false,

  didReceiveAttrs() {
    if (this.isOpen) {
      this._repositionDropdown()
    }
  },

  onDocumentScroll() {
    this._repositionDropdown()
  },

  onDocumentResize() {
    this._repositionDropdown()
  },

  _repositionDropdown() {
    if (!this.isFixed) {
      return;
    }

    let parentElement = this.getDropdownNode()
    let { top, height, width, left } = parentElement.getBoundingClientRect()

    if (this.positionFromTop || this.positionFromRight) {
      throw new Error('`positionFromTop` and `positionFromRight` is incompatible with fixed positioning')
    }

    set(this, 'style', htmlSafe(`
      position: fixed;
      top: ${top + height}px;
      left: ${left}px;
      width: auto;
      min-width: ${width}px
    `));
  },
})
